import { Component } from 'react';
import type { KeyBinding } from '@codemirror/view';
import { StreamLanguage } from '@codemirror/language';
import type { SourceCodeEditorProps } from './props';
/**
---
category: components
---
**/
declare class SourceCodeEditor extends Component<SourceCodeEditorProps> {
    static readonly componentId = "SourceCodeEditor";
    static propTypes: import("@instructure/shared-types").PropValidators<keyof {
        label: string;
        language?: "sh" | "js" | "json" | "javascript" | "jsx" | "shell" | "css" | "html" | "markdown" | "yaml" | "yml" | "bash";
        readOnly?: boolean;
        editable?: boolean;
        lineNumbers?: boolean;
        foldGutter?: boolean;
        highlightActiveLineGutter?: boolean;
        highlightActiveLine?: boolean;
        lineWrapping?: boolean;
        autofocus?: boolean;
        spellcheck?: boolean;
        indentOnLoad?: boolean;
        indentWithTab?: boolean;
        indentUnit?: string;
        direction?: "ltr" | "rtl";
        rtlMoveVisually?: boolean;
        defaultValue?: string;
        value?: string;
        onChange?: (value: string) => void;
        onFocus?: () => void;
        onBlur?: () => void;
        attachment?: "bottom" | "top";
        height?: string;
        width?: string;
        elementRef?: (element: HTMLDivElement | null) => void;
        containerRef?: (element: HTMLDivElement | null) => void;
        searchConfig?: import("./SearchPanel").SearchConfig;
    }>;
    static allowedProps: readonly (keyof {
        label: string;
        language?: "sh" | "js" | "json" | "javascript" | "jsx" | "shell" | "css" | "html" | "markdown" | "yaml" | "yml" | "bash";
        readOnly?: boolean;
        editable?: boolean;
        lineNumbers?: boolean;
        foldGutter?: boolean;
        highlightActiveLineGutter?: boolean;
        highlightActiveLine?: boolean;
        lineWrapping?: boolean;
        autofocus?: boolean;
        spellcheck?: boolean;
        indentOnLoad?: boolean;
        indentWithTab?: boolean;
        indentUnit?: string;
        direction?: "ltr" | "rtl";
        rtlMoveVisually?: boolean;
        defaultValue?: string;
        value?: string;
        onChange?: (value: string) => void;
        onFocus?: () => void;
        onBlur?: () => void;
        attachment?: "bottom" | "top";
        height?: string;
        width?: string;
        elementRef?: (element: HTMLDivElement | null) => void;
        containerRef?: (element: HTMLDivElement | null) => void;
        searchConfig?: import("./SearchPanel").SearchConfig;
    })[];
    static defaultProps: {
        language: string;
        readOnly: boolean;
        editable: boolean;
        lineNumbers: boolean;
        foldGutter: boolean;
        highlightActiveLine: boolean;
        highlightActiveLineGutter: boolean;
        lineWrapping: boolean;
        autofocus: boolean;
        spellcheck: boolean;
        rtlMoveVisually: boolean;
        indentOnLoad: boolean;
        indentWithTab: boolean;
        defaultValue: string;
        height: string;
    };
    private readonly _id;
    ref: HTMLDivElement | null;
    private _containerRef?;
    private _editorView?;
    private _raf;
    private _newSelectionAfterValueChange?;
    handleRef: (el: HTMLDivElement | null) => void;
    handleContainerRef: (el: HTMLDivElement | null) => void;
    private addAnimationFrame;
    private cancelAnimationFrames;
    focus(): void;
    get hasFocus(): boolean | undefined;
    selectAll(): void;
    deselectAll(): void;
    indentCurrentSelection(): void;
    indentAll(): void;
    private indentCodeRange;
    private dispatchViewEffects;
    private dispatchViewChanges;
    private dispatchViewSelection;
    get currentDocValue(): import("@codemirror/state").Text | undefined;
    get isControlled(): boolean;
    constructor(props: SourceCodeEditorProps);
    componentDidMount(): void;
    componentWillUnmount(): void;
    componentDidUpdate(prevProps: SourceCodeEditorProps): void;
    private shouldUpdateExtensions;
    get direction(): "ltr" | "rtl" | undefined;
    get extensions(): (import("@codemirror/state").Extension | StreamLanguage<unknown>)[];
    get baseExtensions(): import("@codemirror/state").Extension[];
    get keymaps(): KeyBinding[];
    get commandKeybinding(): readonly KeyBinding[];
    get themeExtension(): import("@codemirror/state").Extension[] | undefined;
    get languageExtension(): import("@codemirror/language").LanguageSupport | StreamLanguage<unknown>;
    callOnChangeHandler(newValue: string): void;
    get onChangeExtension(): import("@codemirror/state").Extension;
    get focusListenerExtension(): import("@codemirror/state").Extension;
    get announceLineNumberExtension(): import("@codemirror/state").Extension;
    refreshExtensions(): void;
    refreshEditorValue(): void;
    assignAriaLabel: () => void;
    render(): import("@emotion/react/jsx-runtime").JSX.Element;
}
export default SourceCodeEditor;
export { SourceCodeEditor };
//# sourceMappingURL=index.d.ts.map